/* ChessTask - A graphical frontend for creating chess tasks in
*              LaTeX or HTML.
* Copyright (C) 2002-2004 by Dirk Baechle (dl9obn@darc.de)
*
* http://chesstask.sourceforge.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the 
*
* Free Software Foundation, Inc.
* 675 Mass Ave
* Cambridge
* MA 02139
* USA
*
*/

/** 
\file ctTask.cpp
\author Dirk Baechle
\date 2004-03-24
\version 2.0
*/

/*------------------------------------------------------------- Includes */

#include "ctTask.h"
#include "ctPieces.h"

#include <qregexp.h>

/*------------------------------------------------------------ Functions */

/** Default constructor for the \a ctTask class.
*/
ctTask::ctTask() : 
        QObject(),
	bDiagram(true)
{
  ;
}

/** Copy constructor for the \a ctTask class.
@param elem Task that is copied
*/
ctTask::ctTask(const ctTask &elem) : 
        QObject()
{
  // Copy the board
  Position = elem.Position;
  // Copy the strings
  Title = elem.Title;
  Task = elem.Task;
  Solution = elem.Solution;
  // Copy the status for the output of the diagram
  bDiagram = elem.bDiagram;
}


/** Overloaded copy operator ``=''.
@param elem Task that is copied
*/
ctTask &ctTask::operator= (const ctTask &elem)
{
  if (this == &elem)
    return *this;

  // Copy the board
  Position = elem.Position;

  // Copy the strings
  Title = elem.Title;
  Task = elem.Task;
  Solution = elem.Solution;

  // Copy the status for the output of the diagram
  bDiagram = elem.bDiagram;

  return *this;
}

/** Is called whenever an EPD operation
was detected in the input line. It detects the end
of the operation argument, removes the quotes and
replaces the SAN piece characters, according to the
currently selected language.
@param input String, containing the input line
@param tokStart Position of the found EPD operation
@param sToken The ``move token'' as detected by this function
*/
void ctTask::prepareMoveString(const QString &input, int tokStart, 
                               QString &sToken)
{
  // End of the token
  int tokEnd;

  // Step to the argument of the found EPD operation
  tokStart += 3;
  // Find next semicolon
  tokEnd = input.find(QRegExp(";"), tokStart);
  // Set token
  sToken = input.mid(tokStart, (tokEnd-tokStart));
  // Remove quotes 
  sToken.replace(QRegExp("\""), "");
  // Replace SAN piece characters according to the currently 
  // selected language
  sToken.replace(QRegExp("Q"), tr("Q"));
  sToken.replace(QRegExp("R"), tr("R"));
  sToken.replace(QRegExp("B"), tr("B"));
  sToken.replace(QRegExp("N"), tr("N"));
  // Replace ``#''
  sToken.replace(QRegExp("#"), "\\mate");
  // Enclose token in a ``movecomment'' environment
  sToken.append("}");
  sToken.prepend("\\movecomment{");
}

/** Scans the given input string and derives a diagram
and a task text from the FEN information.
Only the first (=diagram) and the second data field (=White or Black
to move?) are used at the moment.
@param input String, containing the input line
*/
void ctTask::importFEN(QString &input)
{
  // String, representing a ``token''
  QString sToken;
  // Start of the token in the string ``input''
  int tokStart = 0;
  // End of the token in the string ``input''
  int tokEnd = 0;

  // Remove all additional whitespace characters
  input.simplifyWhiteSpace();

  // Find the end of the first token = diagram
  tokEnd = input.find(QRegExp(" "));
  // Set token
  sToken = input.mid(tokStart, (tokEnd-tokStart));
  // Get diagram
  Position.importFEN(sToken);

  // Advance token start pointer
  tokStart = tokEnd+1;
  // Find the end of the second token = white/or black to move?
  tokEnd = input.find(QRegExp(" "), tokStart);
  // Set token
  sToken = input.mid(tokStart, (tokEnd-tokStart));
  if (sToken == "b")
  {
    Task = "1. ... ?";
  }
  else
  {
    Task = "1. ?";
  }  

  // The rest of the data fields are not used at the moment...

}

/** Scans the given input string and derives a diagram,
a task text and a solution from the EPD information.
Only the first (=diagram) and the second data field (=White or Black
to move?) are used at the moment. Additionally, the following EPD
``operations'' are detected: bm (best move), am (avoid move), pm
(predicted move), pv (predicted variant).
@param input String, containing the input line
*/
void ctTask::importEPD(QString &input)
{
  // String, representing a ``token''
  QString sToken;
  // Start of the token in the string ``input''
  int tokStart = 0;
  // End of the token in the string ``input''
  int tokEnd = 0;

  // Remove all additional whitespace characters
  input.simplifyWhiteSpace();

  // Find the end of the first token = diagram
  tokEnd = input.find(QRegExp(" "));
  // Set token
  sToken = input.mid(tokStart, (tokEnd-tokStart));
  // Get diagram
  Position.importFEN(sToken);

  // Advance token start pointer
  tokStart = tokEnd+1;
  // Find the end of the second token = white/or black to move?
  tokEnd = input.find(QRegExp(" "), tokStart);
  // Set token
  sToken = input.mid(tokStart, (tokEnd-tokStart));
  if (sToken == "b")
  {
    Task = "1. ..., ?";
  }
  else
  {
    Task = "1. ?";
  }  

  // Now we scan the input for the operations
  // bm (Best move), am (Avoid move),
  // pm (Predicted move) and pv (Predicted variant).
  // The rest of the EPD operations are not used at the moment...

  tokStart = input.find(QRegExp("bm "));
  if (tokStart >= 0)
  {
    // Get argument string 
    prepareMoveString(input, tokStart, sToken);

    // Set new solution text
    Solution = tr("Best move: ") + sToken;
  }
  else
  {
    tokStart = input.find(QRegExp("pv "));
    if (tokStart >= 0)
    {
      // Get argument string 
      prepareMoveString(input, tokStart, sToken);

      // Set new solution text
      Solution = tr("Variant: ") + sToken;
    }
    else
    {
      tokStart = input.find(QRegExp("pm "));
      if (tokStart >= 0)
      {
        // Get argument string 
        prepareMoveString(input, tokStart, sToken);

        // Set new solution text
        Solution = sToken;
      }
    }
  }

  tokStart = input.find(QRegExp("am "));
  if (tokStart >= 0)
  {
    // Get argument string 
    prepareMoveString(input, tokStart, sToken);

    // Is the solution text not empty?
    if ((Solution.isEmpty() == false) && 
        (Solution.isNull() == false))
    {
      // Add another blank
      Solution += " ";
    }

    // Add ``avoid'' to the solution text
    Solution += tr("(Avoid: ") + sToken + ")";
  }
}

